import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Gdk
from . import custom_keyboard as CustomKb
from . import custom_profiles

KEYCAP_SIZE = 40
rkb = None


def build_key_box(key, color, signal_handler):
    box = Gtk.EventBox()
    if not key.isGhost:
        box.get_style_context().add_class('keycap')
        key.color = color
        box.override_background_color(
            Gtk.StateType.NORMAL,
            color
        )
        label = Gtk.Label()
        label.set_text(key.label)
        box.add(label)
    box.keyx = key.x
    box.keyy = key.y
    if key.label in ['tab', 'lctrl', 'lalt', 'ralt', 'rctrl', '\\']:
        box.set_size_request(KEYCAP_SIZE * 1.5, KEYCAP_SIZE)
    elif key.label == 'capslk':
        box.set_size_request(KEYCAP_SIZE * 1.75, KEYCAP_SIZE)
    elif key.label == 'lshift':  # in ['lshift', 'ret']:
        box.set_size_request(KEYCAP_SIZE * 2.25, KEYCAP_SIZE)
    elif key.label == 'ret':
        box.set_size_request(KEYCAP_SIZE * 2.10, KEYCAP_SIZE)
    elif key.label == 'rshift':
        box.set_size_request(KEYCAP_SIZE * 2.75, KEYCAP_SIZE)
    elif key.label == 'bck\nspc':
        box.set_size_request(KEYCAP_SIZE * 2, KEYCAP_SIZE)
    elif key.label == 'spacebar':
        box.set_size_request(KEYCAP_SIZE * 6, KEYCAP_SIZE)
    elif key.label == CustomKb.kblayouts.INV_GHOST:
        box.set_size_request(0, 0)
        box.set_no_show_all(True)
        box.set_visible(False)
    elif key.label == CustomKb.kblayouts.GHOST:
        box.set_size_request(KEYCAP_SIZE, KEYCAP_SIZE)
    else:
        box.set_size_request(KEYCAP_SIZE, KEYCAP_SIZE)
    box.connect('button-press-event', signal_handler)
    return box


def build_keyrow_box(row, colors, signal_handler, prof_index):
    box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
    for key in row.keylist:
        if not key.isGhost:
            k_col = Gdk.RGBA(
                colors[prof_index][0],
                colors[prof_index][1],
                colors[prof_index][2]
            )
        else:
            k_col = Gdk.RGBA(0, 0, 0)
        keybox = build_key_box(
            key,
            k_col,
            signal_handler
        )
        if not key.isGhost:
            prof_index += 1
        box.pack_start(keybox, True, True, 0)
    return {'row': box, 'prof_index': prof_index}


def build_keyboard_box(box, colors, signal_handler, nrkb):
    global rkb
    rkb = nrkb
    # Empty keyboardBox
    if len(box.get_children()) >= 2:
        box.remove(box.get_children()[1])

    prof_index = 0
    kb_inner_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    for row in rkb.rows:
        rowbox_res = build_keyrow_box(row, colors, signal_handler, prof_index)
        rowbox = rowbox_res['row']
        prof_index = rowbox_res['prof_index']
        kb_inner_box.pack_start(rowbox, True, True, 0)
    box.pack_end(kb_inner_box, True, True, 0)
    kb_inner_box.show_all()
    return box



import gi
gi.require_version("Gtk",  "3.0")
from gi.repository import Gtk,  Gio,  Gdk
import os
import json

HOME = os.environ.get('HOME')
CONFDIR = HOME+('/.config/razercommander')
PROFILESFILE = CONFDIR+('/profiles')

blackProfile = {
    'name': 'Empty',
    'colors':  [[0, 0, 0] for i in range(110)]
}


profiles = [blackProfile, ]

if not os.path.isdir(CONFDIR):
    os.makedirs(CONFDIR)

if not os.path.isfile(PROFILESFILE):
    out = open(PROFILESFILE,  'w')
    out.write(json.dumps(profiles))
    out.close()

profilesF = open(PROFILESFILE, 'r')

try:
    profilesText = profilesF.read()
    profiles = json.loads(profilesText)
    profilesF.close()
except ValueError:
    profilesF.close()
    profilesF = open(PROFILESFILE, 'w')
    profilesF.write(json.dumps(profiles))
    profilesF.close()


def makeProfile(name,  rkb):
    profile = {'name': name,  'colors': []}
    for row in rkb.rows:
        for key in row.keylist:
            if not key.isGhost:
                color = key.color
                profile['colors'].append(
                    [
                        color.red,
                        color.green,
                        color.blue
                    ]
                )
    return profile


def saveProfiles():
    profilesF = open(PROFILESFILE, 'w')
    profilesF.write(json.dumps(profiles))
    profilesF.close()


def getProfile(name):
    for p in profiles:
        if p['name'] == name:
            return p


def addProfile(p):
    if p['name'] and p['colors']:
        for i in profiles:
            if i['name'] == p['name']:
                print('Profile {} already exists'.format(p['name']))
                return False
        profiles.append(p)
        saveProfiles()
        return True
    else:
        print('Invalid profile')
        return False


def removeProfile(name):
    for i in profiles:
        if i['name'] == name:
            profiles.remove(i)
            saveProfiles()
            return True
    else:
        print('Profile {} doesn\'t exist'.format(name))
        return False



import os
import json

HOME = os.environ.get('HOME')
CONFDIR = HOME+('/.config/razercommander/')


def check_path(path):
    complete_path = CONFDIR+path
    if not os.path.isdir(CONFDIR):
        os.makedirs(CONFDIR)
    if not os.path.isfile(complete_path):
        out = open(complete_path, 'w')
        out.write(json.dumps({}))
        out.close()


def load_file(path, default):
    complete_path = CONFDIR+path
    mfile = open(complete_path, 'r')
    to_ret = default
    try:
        mfile_txt = mfile.read()
        to_ret = json.loads(mfile_txt)
        mfile.close()
    except ValueError:
        mfile.close()
        mfile = open(complete_path, 'w')
        mfile.write(json.dumps(default))
        mfile.close()
    finally:
        return to_ret


def save_file(path, value):
    # print(value)
    complete_path = CONFDIR + path
    check_path(path)
    out = open(complete_path, 'w')
    out.write(json.dumps(value))
    out.close()



from . import kblayouts
import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gdk


class RKeyboardKey:
    label = ''
    color = Gdk.RGBA(0, 0, 0)
    isGhost = False
    x = -1
    y = -1

    def __init__(self, label, x, y):
        if label in [kblayouts.GHOST, kblayouts.INV_GHOST]:
            self.isGhost = True
        self.label = label
        self.x = x
        self.y = y

    def __str__(self):
        return self.label + self.color.to_string()


class RKeyboardRow:
    keylist = []
    rowindex = -1

    def __init__(self, rowarr, index):
        self.rowindex = index
        self.keylist = []
        cx = 0
        for key in rowarr:
            self.keylist.append(RKeyboardKey(key, cx, index))
            cx += 1

    def getKey(self, index):
        return self.keylist[index]


class RKeyboard:
    layout = 'ansi_us'
    layoutList = None
    rows = []

    def __init__(self, layout):
        # self.layout=layout
        self.layout = 'ansi_us'
        self.layoutList = kblayouts.layouts[layout]
        index = 0
        self.rows = []
        for row in self.layoutList:
            self.rows.append(RKeyboardRow(row, index))
            index += 1

    def getKey(self, x, y):
        row = self.rows[y]
        key = row.getKey(x)
        return key



# __main__.py
#
# Copyright (C) 2017 GabMus
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import getpass
import grp


def check_plugdev():
    """Check if group 'plugdev' exists and current user is its member.
    If razerCommander is being run from within a flatpak, skip this check.
    There is apparently no way to check groups with flatpak confinement."""
    if os.path.isfile('{}/flatpak-info'.format(os.environ['XDG_RUNTIME_DIR'])):
        return
    try:
        if not getpass.getuser() in grp.getgrnam('plugdev').gr_mem:
            print('''
ERROR: you are not part of the plugdev group. Add yourself to it:
  $ sudo gpasswd -a <YOUR_USERNAME> plugdev
''')
            exit(1)
    except KeyError:
        print('''
ERROR: the plugdev group doesn\'t exist. Make sure you installed the openrazer
driver, in case you did, create the plugdev group and add yourself to it:
  $ sudo groupadd plugdev
  $ sudo gpasswd -a <YOUR_USERNAME> plugdev
''')
        exit(1)


check_plugdev()

import argparse
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, Gio, GdkPixbuf
print('Importing device logic, waiting for daemon')

try:
    from . import device
    print('Device logic loaded, daemon is alive')
except Exception as e:
    print('ERROR: the daemon is not responding!\nTry running `killall razer-daemon && razer-daemon` or rebooting. If this doesn\'t work, please fill an issue!')
    print('Exception: {}'.format(e))
    exit(1)
from . import custom_keyboard as CustomKb
from . import custom_profiles
from . import listboxHelper
from . import custom_kb_builder

HOME = os.environ.get('HOME')


class Application(Gtk.Application):
    def __init__(self, **kwargs):
        self.builder = Gtk.Builder.new_from_resource(
            '/org/gabmus/razercommander/ui/ui.glade'
        )
        super().__init__(
            application_id='org.gabmus.razercommander',
            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
            **kwargs
        )
        self.RESOURCE_PATH = '/org/gabmus/razercommander/'

        self.builder.connect_signals(self)

        settings = Gtk.Settings.get_default()
        settings.set_property("gtk-application-prefer-dark-theme", True)

        self.window = self.builder.get_object('window')

        self.brightnessScale = self.builder.get_object('brightnessScale')
        self.logoBrightnessScale = self.builder.get_object('logoBrightnessScale')
        self.scrollBrightnessScale = self.builder.get_object('scrollBrightnessScale')
        self.brightnessBox = self.builder.get_object('brightnessBox')
        self.mouseBrightnessBox = self.builder.get_object('mouseBrightnessBox')
        self.currentDeviceLabel = self.builder.get_object('currentDeviceLabel')
        self.customColorPicker = self.builder.get_object('customColorPicker')
        self.fxListBox = self.builder.get_object('fxListBox')
        self.gameModeIcon = self.builder.get_object('gameModeIcon')
        self.gameModeSwitch = self.builder.get_object("gameModeSwitch")
        self.keyboardBox = self.builder.get_object("keyboardBox")
        self.macro_current_keystroke_label = self.builder.get_object('currentKeystrokeLabel')
        self.macro_device_picture = self.builder.get_object('macroDevicePicture')
        self.macro_shortcut_dialog = self.builder.get_object('macroShortcutDialog')
        self.macro_shortcut_entry = self.builder.get_object('macroShortcutEntry')
        self.macro_shortcut_dialog_keyname_label = self.builder.get_object('macroShortcutDialogKeyName')
        self.macro_listbox = self.builder.get_object('macroShortcutsListbox')
        self.mainBox = self.builder.get_object('mainBox')
        self.mainStack = self.builder.get_object('mainStack')
        self.mainStackSwitcherButtons = self.builder.get_object('mainStackSwitcherButtons')
        self.noDevicesLabel = self.builder.get_object('noDevicesLabel')
        self.pipetteTBtn = self.builder.get_object('customPipetteToolBtn')
        self.popoverChooseDevice = self.builder.get_object('popoverChooseDevice')
        self.popoverDevicesListBox = self.builder.get_object('popoverDevicesListBox')
        self.popoverProfiles = self.builder.get_object('popoverProfiles')
        self.profileNameEntry = self.builder.get_object('profileNameEntry')
        self.profilesListBox = self.builder.get_object('popoverProfilesListBox')
        self.saveProfileDialog = self.builder.get_object('saveProfileDialog')
        self.set_shortcut_stack = self.builder.get_object('setShortcutStack')
        self.universalApplyButton = self.builder.get_object('universalApplyButton')

        self.syncFXBox = self.builder.get_object('syncFXBox')
        self.syncFXToggle = self.builder.get_object('syncFXToggle')

        self.breathDoubleRadio = self.builder.get_object('breathDoubleRadio')
        self.breathRandomRadio = self.builder.get_object('breathRandomRadio')
        self.breathRGB1 = self.builder.get_object('breathRGB1chooser')
        self.breathRGB2 = self.builder.get_object('breathRGB2chooser')
        self.breathSingleRadio = self.builder.get_object('breathSingleRadio')
        self.reactiveRGBchooser = self.builder.get_object('reactiveRGBchooser')
        self.rippleColorPicker = self.builder.get_object('rippleColorPicker')
        self.rippleRadioColor = self.builder.get_object('rippleRadioColor')
        self.rippleRadioRandom = self.builder.get_object('rippleRadioRandom')
        self.spinReactiveTime = self.builder.get_object('spinReactiveTime')
        self.staticRGB = self.builder.get_object('staticRGBchooser')
        self.waveToggleLeft = self.builder.get_object('waveToggleBtnLeft')
        self.waveToggleRight = self.builder.get_object('waveToggleBtnRight')
        self.blinkRGBChooser = self.builder.get_object('blinkRGBChooser')
        self.pulsateRGBChooser = self.builder.get_object('pulsateRGBChooser')

        self.breathSettingsBox = self.builder.get_object('breathSettingsBox')
        self.waveSettingsBox = self.builder.get_object('waveSettingsBox')
        self.staticSettingsBox = self.builder.get_object('staticSettingsBox')
        self.reactiveSettingsBox = self.builder.get_object('reactiveSettingsBox')
        self.rippleSettingsBox = self.builder.get_object('rippleSettingsBox')
        self.blinkSettingsBox = self.builder.get_object('blinkSettingsBox')
        self.pulsateSettingsBox = self.builder.get_object('pulsateSettingsBox')
        self.clearTBtn = self.builder.get_object('customClearToolBtn')
        self.fxStack = self.builder.get_object('fxStack')

        self.dpi1Adjustment = self.builder.get_object('mouseDpi1Adjustment')
        self.dpi2Adjustment = self.builder.get_object('mouseDpi2Adjustment')
        self.radioPollrateGroup = self.builder.get_object(
            'radioPollrate125'
        ).get_group()

        self.radioPollrate125 = self.builder.get_object('radioPollrate125')
        self.radioPollrate500 = self.builder.get_object('radioPollrate500')
        self.radioPollrate1000 = self.builder.get_object('radioPollrate1000')

        self.settingsPanes = {
            'Breath': self.breathSettingsBox,
            'Wave': self.waveSettingsBox,
            'Static': self.staticSettingsBox,
            'Reactive': self.reactiveSettingsBox,
            'Custom': self.keyboardBox,
            'Ripple': self.rippleSettingsBox,
            'Scroll blinking': self.blinkSettingsBox,
            'Logo blinking': self.blinkSettingsBox,
            'Scroll pulsate': self.pulsateSettingsBox,
            'Logo pulsate': self.pulsateSettingsBox,
            'Scroll breath': self.breathSettingsBox,
            'Logo breath': self.breathSettingsBox,
            'Scroll reactive': self.reactiveSettingsBox,
            'Logo reactive': self.reactiveSettingsBox,
            'Scroll static': self.staticSettingsBox,
            'Logo static': self.staticSettingsBox,
        }

        self.settingsPanesNames = {
            'Breath': 'breathSettingsBox',
            'Wave': 'waveSettingsBox',
            'Static': 'staticSettingsBox',
            'Reactive': 'reactiveSettingsBox',
            'Custom': 'keyboardSettingsBox',
            'Ripple': 'rippleSettingsBox',
            'None': 'noneSettingsBox',
            'Spectrum': 'spectrumSettingsBox',
            'Scroll blinking': 'blinkSettingsBox',
            'Logo blinking': 'blinkSettingsBox',
            'Scroll pulsate': 'pulsateSettingsBox',
            'Logo pulsate': 'pulsateSettingsBox',
            'Scroll breath': 'breathSettingsBox',
            'Logo breath': 'breathSettingsBox',
            'Scroll reactive': 'reactiveSettingsBox',
            'Logo reactive': 'reactiveSettingsBox',
            'Scroll static': 'staticSettingsBox',
            'Logo static': 'staticSettingsBox',
            'Scroll spectrum': 'spectrumSettingsBox',
            'Logo spectrum': 'spectrumSettingsBox',
            'Logo none': 'noneSettingsBox',
            'Scroll none': 'noneSettingsBox',
        }

        self.setAnimation('breathAnimIcon', 'breath')
        self.setAnimation('customAnimIcon', 'custom')
        self.setAnimation('noneAnimIcon', 'none')
        self.setAnimation('reactiveAnimIcon', 'reactive')
        self.setAnimation('rippleAnimIcon', 'ripple')
        self.setAnimation('spectrumAnimIcon', 'spectrum')
        self.setAnimation('staticAnimIcon', 'static')
        self.setAnimation('waveAnimIcon', 'wave')

        self.rkb = CustomKb.RKeyboard('ansi_us')

        self.KEYCAP_SIZE = 50
        self.keystroke_shortcuts_all_mods_str = [
            'Shift_L',
            'Control_L',
            'Super_L',
            'Alt_L'
        ]

        self.keystroke_shortcuts_mod_shift = self.builder.get_object(
            'setShortcutKeystrokeShiftModifierCheck'
        )
        self.keystroke_shortcuts_mod_ctrl = self.builder.get_object(
            'setShortcutKeystrokeCtrlModifierCheck'
        )
        self.keystroke_shortcuts_mod_super = self.builder.get_object(
            'setShortcutKeystrokeSuperModifierCheck'
        )
        self.keystroke_shortcuts_mod_alt = self.builder.get_object(
            'setShortcutKeystrokeAltModifierCheck'
        )
        self.keystroke_shortcuts_all_mods = [
            self.keystroke_shortcuts_mod_shift,
            self.keystroke_shortcuts_mod_ctrl,
            self.keystroke_shortcuts_mod_super,
            self.keystroke_shortcuts_mod_alt
        ]

        self.devicesList = []
        self.active_razer_device = None

        self.key_stroke_n = 0
        self.key_stroke_list = []

    def initDevices(self):
        self.devicesList = []
        for dev in device.devlist:
            try:
                newdev = device.Device(dev)
                self.devicesList.append(newdev)
            except Exception as ex:
                print('Skipping device {}\n  Exception: {}'.format(dev.name, ex))
                pass
        if len(self.devicesList) > 0:
            self.active_razer_device = self.devicesList[0]
        else:
            self.active_razer_device = None

    def setAnimation(self, widget_name, anim_name):
        self.builder.get_object(
            widget_name).set_from_animation(
                GdkPixbuf.PixbufAnimation.new_from_resource(
                    '{}animations/{}.gif'.format(self.RESOURCE_PATH, anim_name)
                )
            )

    def showOrHideSyncFX(self):
        if len(self.devicesList) > 1:
            self.syncFXBox.set_visible(True)
            self.syncFXToggle.set_state(device.getSyncFX())
        else:
            self.syncFXBox.set_visible(False)

    def fillDevicesList(self):
        if len(self.devicesList) > 0:
            for i in self.devicesList:
                row = listboxHelper.make_row(i.name)
                row.value = i
                self.popoverDevicesListBox.add(row)
            self.popoverDevicesListBox.select_row(
                self.popoverDevicesListBox.get_row_at_index(0)
            )
            self.currentDeviceLabel.set_text(
                self.popoverDevicesListBox.get_row_at_index(0).value.name
            )
            self.showOrHideSyncFX()
        else:
            self.currentDeviceLabel.set_text('No devices')

    def updateDevicesConnected(self):
        if len(self.devicesList) > 0:
            self.mainBox.show_all()
            self.keyboardBox.hide()
            self.noDevicesLabel.hide()
            self.gameModeSwitch.set_sensitive(True)
            self.universalApplyButton.set_sensitive(True)
        else:
            print("no devices")
            self.mainBox.hide()
            self.noDevicesLabel.show()
            self.gameModeSwitch.set_sensitive(False)
            self.universalApplyButton.set_sensitive(False)

    def refreshDevices(self):
        listboxHelper.empty_listbox(self.popoverDevicesListBox)
        self.initDevices()
        self.fillDevicesList()
        self.updateDevicesConnected()

    def refreshProfiles(self):
        # empty list first
        for child in self.profilesListBox.get_children():
            self.profilesListBox.remove(child)

        for p in custom_profiles.profiles:
            box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
            labelName = Gtk.Label()
            labelName.set_text(p['name'])
            box.pack_start(labelName, True, True, 0)
            box.set_margin_top(6)
            box.set_margin_bottom(6)
            if not p['name'] == 'Empty':
                rmIcon = Gtk.Image()
                rmIcon.set_from_icon_name('gtk-delete', Gtk.IconSize.BUTTON)
                rmButton = Gtk.Button()
                rmButton.add(rmIcon)
                rmButton.preset = p['name']
                rmButton.connect("button-press-event", self.onRmProfile)
                box.pack_end(rmButton, False, False, 0)
            row = Gtk.ListBoxRow()
            row.add(box)
            row.value = p['name']
            self.profilesListBox.add(row)
        self.profilesListBox.unselect_all()
        if self.popoverProfiles.get_visible():
            self.popoverProfiles.show_all()

    def onRmProfile(self, button, smth):
        custom_profiles.removeProfile(button.preset)
        self.refreshProfiles()

    def refresh_macro_section(self):
        if not self.active_razer_device or not self.active_razer_device.macro_device:
            return
        listboxHelper.empty_listbox(self.macro_listbox)
        rows_l = device.macro_logic.make_shortcutslistbox_rows(self.active_razer_device.macro_device)
        for row in rows_l:
            self.macro_listbox.add(row)
            row.show_all()
        self.macro_device_picture.set_from_resource(
            '{}img/{}.svg'.format(
                self.RESOURCE_PATH,
                self.active_razer_device.name.replace(' ', '_')
            )
        )

    def refreshFxList(self):
        if not self.active_razer_device:
            return
        # empty list before (re)filling it
        listboxHelper.empty_listbox(self.fxListBox)
        # fill list with supported effects
        if self.active_razer_device.device.type == 'mouse' and self.active_razer_device.device.fx.has('scroll_brightness'):
            self.brightnessBox.hide()
            self.mouseBrightnessBox.show_all()
            self.logoBrightnessScale.set_value(
                self.active_razer_device.getLogoBrightness()
            )
            self.scrollBrightnessScale.set_value(
                self.active_razer_device.getScrollBrightness()
            )
        elif self.active_razer_device.device.type == 'keyboard':
            self.brightnessBox.show_all()
            self.brightnessScale.set_value(
                self.active_razer_device.getKbBrightness()
            )
            self.mouseBrightnessBox.hide()
        else:
            self.mouseBrightnessBox.hide()
            self.brightnessBox.hide()
        for i in self.active_razer_device.availableFX:
            if i not in [
                'breath_single',
                'breath_dual',
                'scroll_breath_single',
                'scroll_breath_dual',
                'logo_breath_single',
                'logo_breath_dual'
            ]:
                if i == 'breath_random':
                    i = 'breath'

                if i == 'scroll_breath_random':
                    i = 'scroll_breath'
                elif i == 'logo_breath_random':
                    i = 'logo_breath'

                if 'scroll' in i or 'logo' in i:
                    i = i.replace('_', ' ')

                i = i.capitalize()

                iconPath = '{}img/{}.svg'.format(
                    self.RESOURCE_PATH, i.replace(' ', '_')
                )
                row = listboxHelper.make_image_row(
                    i,
                    iconPath
                )
                row.value = i
                self.fxListBox.add(row)
                row.show_all()
        if self.active_razer_device.device.has('game_mode_led'):
            self.gameModeSwitch.set_state(
                self.active_razer_device.device.game_mode_led
            )
            if self.gameModeSwitch.get_state():
                self.gameModeIcon.set_from_resource(
                    '{}img/gameModeOn.svg'.format(self.RESOURCE_PATH)
                )
            else:
                self.gameModeIcon.set_from_resource(
                    '{}img/gameModeOff.svg'.format(self.RESOURCE_PATH)
                )
            self.gameModeIcon.show()
            self.gameModeSwitch.show()
        else:
            self.gameModeIcon.hide()
            self.gameModeSwitch.hide()

        # selective hiding of switcher buttons
        stackButtons = self.mainStackSwitcherButtons.get_children()

        # if device supports macros
        if self.active_razer_device.macro_device:
            stackButtons[2].show()
            self.refresh_macro_section()  # possibly move this from here
        else:
            stackButtons[2].hide()
        if self.active_razer_device.device.type == 'mouse':
            stackButtons[0].show()
            self.builder.get_object(
                'mouseDpi1Adjustment'
                ).set_upper(self.active_razer_device.get_max_dpi())
            self.builder.get_object(
                'mouseDpi2Adjustment'
                ).set_upper(self.active_razer_device.get_max_dpi())
            currentDpi = self.active_razer_device.get_dpi()
            print(currentDpi[1])
            self.builder.get_object(
                'mouseDpi1Adjustment'
                ).set_value(currentDpi[0])
            self.builder.get_object(
                'mouseDpi2Adjustment'
                ).set_value(currentDpi[1])
            if self.active_razer_device.get_poll_rate() == 125:
                self.radioPollrate125.set_active(True)
            elif self.active_razer_device.get_poll_rate() == 500:
                self.radioPollrate500.set_active(True)
            elif self.active_razer_device.get_poll_rate() == 1000:
                self.radioPollrate1000.set_active(True)
            else:
                self.radioPollrate500.set_active(True)
        else:
            stackButtons[0].hide()
            self.mainStack.set_visible_child(self.mainStack.get_children()[1])
        # fxListBox.show_all()

    def onVirtKeyClick(self, eventbox, eventbtn):
        key = self.rkb.getKey(eventbox.keyx, eventbox.keyy)
        if not key.isGhost:
            if self.pipetteTBtn.get_active():
                color = key.color
                self.customColorPicker.set_rgba(color)
                self.pipetteTBtn.set_active(False)
            elif self.clearTBtn.get_active():
                key.color = Gdk.RGBA(0, 0, 0)
                black_rgba = Gdk.RGBA(0, 0, 0)
                eventbox.override_background_color(
                    Gtk.StateType.NORMAL,
                    black_rgba
                )
            else:
                key.color = self.customColorPicker.get_rgba()
                eventbox.override_background_color(
                    Gtk.StateType.NORMAL,
                    self.customColorPicker.get_rgba()
                )

    def drawKB(self, profile=None):
        # Load black fake profile if none provided
        if not profile:
            profile = custom_profiles.blackProfile
        custom_kb_builder.build_keyboard_box(
            self.keyboardBox,
            profile['colors'],
            self.onVirtKeyClick,
            self.rkb
        )

    def getColorVal(self, n):
        return int(n * 255)

    def enableFXwSettings(self, fx):
        if fx == 'Breath':
            if self.breathRandomRadio.get_active():
                self.active_razer_device.enableRandomBreath()
            else:
                rgb1 = self.breathRGB1.get_rgba()
                r1 = self.getColorVal(rgb1.red)
                g1 = self.getColorVal(rgb1.green)
                b1 = self.getColorVal(rgb1.blue)
                if self.breathDoubleRadio.get_active():
                    rgb2 = self.breathRGB2.get_rgba()
                    r2 = self.getColorVal(rgb2.red)
                    g2 = self.getColorVal(rgb2.green)
                    b2 = self.getColorVal(rgb2.blue)
                    self.active_razer_device.enableDoubleBreath(
                        r1, g1, b1, r2, g2, b2
                    )
                else:
                    self.active_razer_device.enableSingleBreath(r1, g1, b1)
        elif fx == 'Wave':
            if self.waveToggleLeft.get_active():
                self.active_razer_device.enableWave(2)
            else:
                self.active_razer_device.enableWave(1)
        elif fx == 'Static':
            rgb = self.staticRGB.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableStatic(r, g, b)
        elif fx == 'Reactive':
            r_time = self.spinReactiveTime.get_value_as_int()
            rgb = self.reactiveRGBchooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableReactive(r_time, r, g, b)
        elif fx == 'Custom':
            self.active_razer_device.applyCustom(self.rkb)
        elif fx == 'Ripple':
            if self.rippleRadioRandom.get_active():
                self.active_razer_device.enableRippleRandom()
            else:
                rgb = self.rippleColorPicker.get_rgba()
                r = self.getColorVal(rgb.red)
                g = self.getColorVal(rgb.green)
                b = self.getColorVal(rgb.blue)
                self.active_razer_device.enableRipple(r, g, b)
        elif fx == 'Scroll blinking':
            rgb = self.blinkRGBChooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableScrollBlinking(r, g, b)
        elif fx == 'Logo blinking':
            rgb = self.blinkRGBChooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableLogoBlinking(r, g, b)
        elif fx == 'Scroll pulsate':
            rgb = self.blinkRGBChooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableScrollPulsate(r, g, b)
        elif fx == 'Logo pulsate':
            rgb = self.blinkRGBChooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableLogoPulsate(r, g, b)
        elif fx == 'Scroll breath':
            if self.breathRandomRadio.get_active():
                self.active_razer_device.enableScrollBreathRandom()
            else:
                rgb1 = self.breathRGB1.get_rgba()
                r1 = self.getColorVal(rgb1.red)
                g1 = self.getColorVal(rgb1.green)
                b1 = self.getColorVal(rgb1.blue)
                if self.breathDoubleRadio.get_active():
                    rgb2 = self.breathRGB2.get_rgba()
                    r2 = self.getColorVal(rgb2.red)
                    g2 = self.getColorVal(rgb2.green)
                    b2 = self.getColorVal(rgb2.blue)
                    self.active_razer_device.enableScrollBreathDual(
                        r1, g1, b1, r2, g2, b2
                    )
                else:
                    self.active_razer_device.enableScrollBreathSingle(
                        r1, g1, b1
                    )
        elif fx == 'Logo breath':
            if self.breathRandomRadio.get_active():
                self.active_razer_device.enableLogoBreathRandom()
            else:
                rgb1 = self.breathRGB1.get_rgba()
                r1 = self.getColorVal(rgb1.red)
                g1 = self.getColorVal(rgb1.green)
                b1 = self.getColorVal(rgb1.blue)
                if self.breathDoubleRadio.get_active():
                    rgb2 = self.breathRGB2.get_rgba()
                    r2 = self.getColorVal(rgb2.red)
                    g2 = self.getColorVal(rgb2.green)
                    b2 = self.getColorVal(rgb2.blue)
                    self.active_razer_device.enableLogoBreathDual(
                        r1, g1, b1, r2, g2, b2
                    )
                else:
                    self.active_razer_device.enableLogoBreathSingle(r1, g1, b1)
        elif fx == 'Scroll reactive':
            r_time = self.spinReactiveTime.get_value_as_int()
            rgb = self.reactiveRGBchooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableScrollReactive(r, g, b, r_time)
        elif fx == 'Logo reactive':
            r_time = self.spinReactiveTime.get_value_as_int()
            rgb = self.reactiveRGBchooser.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableLogoReactive(r, g, b, r_time)
        elif fx == 'Scroll static':
            rgb = self.staticRGB.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableScrollStatic(r, g, b)
        elif fx == 'Logo static':
            rgb = self.staticRGB.get_rgba()
            r = self.getColorVal(rgb.red)
            g = self.getColorVal(rgb.green)
            b = self.getColorVal(rgb.blue)
            self.active_razer_device.enableLogoStatic(r, g, b)
        else:
            self.active_razer_device.enableFX(fx)

    def do_activate(self):
        window = self.builder.get_object('window')
        self.add_window(window)
        window.set_wmclass('razerCommander', 'razerCommander')
        window.set_title('razerCommander')

        appMenu = Gio.Menu()
        appMenu.append("About", "app.about")
        appMenu.append("Quit", "app.quit")
        about_action = Gio.SimpleAction.new("about", None)
        about_action.connect("activate", self.on_about_activate)
        self.builder.get_object("aboutdialog").connect(
            "delete-event", lambda *_:
                self.builder.get_object("aboutdialog").hide() or True
        )
        self.add_action(about_action)
        quit_action = Gio.SimpleAction.new("quit", None)
        quit_action.connect("activate", self.on_quit_activate)
        self.add_action(quit_action)
        self.set_app_menu(appMenu)

        # initialization protocols
        self.refreshDevices()
        self.refreshProfiles()
        self.keyboardBox.hide()
        self.updateDevicesConnected()
        # Quit if we only need initialization
        if self.args.quit_after_init:
            self.quit()

        window.show_all()
        self.updateDevicesConnected()
        self.refreshFxList()
        self.keyboardBox.hide()

    def do_command_line(self, args):
        '''
        GTK.Application command line handler
        called if Gio.ApplicationFlags.HANDLES_COMMAND_LINE is set.
        must call the self.do_activate() to get the application up and running.
        '''
        Gtk.Application.do_command_line(self, args)  # call the default commandline handler
        # make a command line parser
        parser = argparse.ArgumentParser(prog='gui')
        # add a -c/--color option
        parser.add_argument('-q', '--quit-after-init', dest='quit_after_init', action='store_true', help='initialize application (e.g. for macros initialization on system startup) and quit')
        # parse the command line stored in args, but skip the first element (the filename)
        self.args = parser.parse_args(args.get_arguments()[1:])
        # call the main program do_activate() to start up the app
        self.do_activate()
        return 0

    def on_about_activate(self, *args):
        self.builder.get_object("aboutdialog").show()

    def on_quit_activate(self, *args):
        self.quit()

    def onDeleteWindow(self, *args):
        self.quit()

    # Handler functions START

    def on_refreshDevicesButton_clicked(self, button):
        self.refreshDevices()

    def on_aboutdialog_close(self, *args):
        self.builder.get_object("aboutdialog").hide()

    def on_customProfilesButton_clicked(self, button):
        if not self.popoverProfiles.get_visible():
            self.popoverProfiles.show_all()

    def on_popoverProfilesListBox_row_selected(self, list, row):
        if row:
            profile = custom_profiles.getProfile(row.value)
            self.drawKB(profile)
            self.popoverProfiles.hide()

    def on_popoverProfilesSaveProfile_clicked(self, button):
        self.saveProfileDialog.show_all()
        self.popoverProfiles.hide()

    def on_saveProfileDialogOk_clicked(self, button):
        name = self.profileNameEntry.get_text()
        if not custom_profiles.addProfile(
            custom_profiles.makeProfile(name, self.rkb)
        ):
            print('error')
            # presetExistsInfobar.show()
        else:
            # presetExistsInfobar.hide()
            self.profileNameEntry.set_text('')
            self.saveProfileDialog.hide()
            self.refreshProfiles()

    def on_presetExistsInfobar_close(self, btn, infobar):
        infobar.hide()

    def on_saveProfileDialogCancel_clicked(self, button):
        self.saveProfileDialog.hide()
        self.profileNameEntry.set_text('')

    def on_deviceChooserButton_clicked(self, button):
        if self.popoverChooseDevice.get_visible():
            self.popoverChooseDevice.hide()
        else:
            self.popoverChooseDevice.show_all()

    def on_popoverDevicesListBox_row_selected(self, list, row):
        # TODO ?: move to changeDevice() function
        if row:
            self.active_razer_device = row.value
            self.currentDeviceLabel.set_text(row.value.name)
            self.fxStack.set_visible_child(
                self.fxStack.get_child_by_name('defaultSettingsBox')
            )
            self.refreshFxList()
            self.popoverChooseDevice.hide()

    def on_universalApplyButton_clicked(self, button):
        if self.mainStack.get_visible_child_name() == 'Mouse':
            self.active_razer_device.set_dpi(
                int(self.dpi1Adjustment.get_value()),
                int(self.dpi2Adjustment.get_value()),
            )
            currentPollRate = 500
            if self.radioPollrate125.get_active():
                currentPollRate = 125
            elif self.radioPollrate500.get_active():
                currentPollRate = 500
            elif self.radioPollrate1000.get_active():
                currentPollRate = 1000
            self.active_razer_device.set_poll_rate(currentPollRate)
            return

        if self.active_razer_device.device.type == 'mouse' and self.active_razer_device.device.fx.has('scroll_brightness'):
            logoBrightness = self.logoBrightnessScale.get_value()
            scrollBrightness = self.scrollBrightnessScale.get_value()
            self.active_razer_device.setLogoBrightness(int(logoBrightness))
            self.active_razer_device.setScrollBrightness(int(scrollBrightness))
        elif self.active_razer_device.device.has('brightness'):
            newVal = self.brightnessScale.get_value()
            self.active_razer_device.setBrightness(int(newVal))
        if not self.fxListBox.get_selected_row():
            print('No fx row selected')
            return
        currentFX = self.fxListBox.get_selected_row().value
        if currentFX in self.settingsPanes.keys():
            self.enableFXwSettings(currentFX)
        else:
            self.active_razer_device.enableFX(currentFX)

    def on_gameModeSwitch_state_set(self, *args):
        self.active_razer_device.toggleGameMode()
        value = self.gameModeSwitch.get_state()
        # the state is inverted
        if not value:
            self.gameModeIcon.set_from_resource('{}img/gameModeOn.svg'.format(self.RESOURCE_PATH))
        else:
            self.gameModeIcon.set_from_resource('{}img/gameModeOff.svg'.format(self.RESOURCE_PATH))

    def on_syncFXToggle_state_set(self, *args):
        value = self.syncFXToggle.get_state()
        # the state is inverted
        device.setSyncFX(not value)

    def on_breathRadio_toggled(self, radio):
        if radio.get_state():
            rlabel = radio.get_label()
            if rlabel == "Random":
                self.breathRGB1.set_sensitive(False)
                self.breathRGB2.set_sensitive(False)
            elif rlabel == "Single color":
                self.breathRGB1.set_sensitive(True)
                self.breathRGB2.set_sensitive(False)
            elif rlabel == "Double color":
                self.breathRGB1.set_sensitive(True)
                self.breathRGB2.set_sensitive(True)

    def on_rippleRadioColor_toggled(self, radio):
        if radio.get_state():
            self.rippleColorPicker.set_sensitive(True)
        else:
            self.rippleColorPicker.set_sensitive(False)

    def on_waveToggleBtn_left(self, button):
        if self.waveToggleLeft.get_active():
            self.waveToggleRight.set_active(False)

    def on_waveToggleBtn_right(self, button):
        if self.waveToggleRight.get_active():
            self.waveToggleLeft.set_active(False)

    def on_customPipetteToolBtn_toggled(self, button):
        if self.clearTBtn.get_active() and button.get_active():
            self.clearTBtn.set_active(False)

    def on_customClearToolBtn_toggled(self, button):
        if self.pipetteTBtn.get_active() and button.get_active():
            self.pipetteTBtn.set_active(False)

    def on_customColorPicker_color_set(self, *args):
        if self.pipetteTBtn.get_active():
            self.pipetteTBtn.set_active(False)
        if self.clearTBtn.get_active():
            self.clearTBtn.set_active(False)

    def on_fxListBox_row_selected(self, list, row):
        if row:
            if row.value in self.settingsPanesNames.keys():
                self.builder.get_object(self.settingsPanesNames[row.value]).show_all()
                self.fxStack.set_visible_child(
                    self.fxStack.get_child_by_name(
                        self.settingsPanesNames[row.value]
                    )
                )
            if row.value == 'Custom':
                self.drawKB()
            else:
                self.keyboardBox.hide()

    def on_macroShortcutDialogCancel_clicked(self, btn):
        self.macro_shortcut_dialog.hide()
        self.macro_shortcut_entry.set_text('')

    def on_macroShortcutsListbox_row_activated(self, list, row):
        if row and row.value:
            # uncheck all modifier checkboxes
            for i in self.keystroke_shortcuts_all_mods:
                i.set_active(False)
            self.macro_current_keystroke_label.set_text('...')
            self.macro_shortcut_dialog_keyname_label.set_text(row.value['key'])
            self.set_shortcut_stack.set_visible_child_name('Command')
            if not row.value['val']:
                self.macro_shortcut_entry.set_text('')
            elif row.value['val'].startswith('xdotool key '):  # is keystroke
                self.set_shortcut_stack.set_visible_child_name('Keystroke')
                current_keystroke = row.value['val'][12:]
                self.macro_current_keystroke_label.set_text(current_keystroke)
            else:
                self.macro_shortcut_entry.set_text(row.value['val'])
            self.macro_shortcut_dialog.show()

    def on_macroShortcutDialogOk_clicked(self, btn):
        if self.set_shortcut_stack.get_visible_child_name() == 'Keystroke':
            n_macro = 'xdotool key {}'.format(self.macro_current_keystroke_label.get_text())
        else:
            n_macro = self.macro_shortcut_entry.get_text()
        macro_d = self.active_razer_device.macro_device
        macro_d.set_macro(
            self.macro_shortcut_dialog_keyname_label.get_text(),
            n_macro
        )
        self.refresh_macro_section()
        self.macro_shortcut_dialog.hide()

    def on_macroShortcutDialogClear_clicked(self, btn):
        macro_d = self.active_razer_device.macro_device
        n_macro = ''
        macro_d.set_macro(
            self.macro_shortcut_dialog_keyname_label.get_text(),
            n_macro
        )
        self.refresh_macro_section()
        self.macro_shortcut_dialog.hide()

    def on_recordKeystrokeToggleBtn_key_press_event(self, toggle_btn, event):
        if toggle_btn.get_active():
            keyname = Gdk.keyval_name(event.keyval)
            if keyname not in self.key_stroke_list:
                self.key_stroke_list.append(keyname)
                self.key_stroke_n += 1
        else:
            self.key_stroke_n = 0
            self.key_stroke_list = []

    def on_recordKeystrokeToggleBtn_key_release_event(self, toggle_btn, event):
        if toggle_btn.get_active():
            keyname = Gdk.keyval_name(event.keyval)
            self.key_stroke_n -= 1
            if not self.key_stroke_n:
                keystroke = ''
                for i in range(0, 4):
                    if self.keystroke_shortcuts_all_mods[i].get_active():
                        keystroke += self.keystroke_shortcuts_all_mods_str[i]+'+'
                keystroke += '+'.join(self.key_stroke_list)
                self.macro_current_keystroke_label.set_text(keystroke)
                self.key_stroke_list = []
                self.key_stroke_n = 0
                toggle_btn.set_active(False)
        else:
            self.key_stroke_n = 0
            self.key_stroke_list = []

    # Handler functions END


def main():
    application = Application()

    try:
        ret = application.run(sys.argv)
    except SystemExit as e:
        ret = e.code

    sys.exit(ret)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import subprocess
from os import path
from tempfile import TemporaryDirectory

manifest = sys.argv[1]
output = sys.argv[2]
app_id = path.basename(manifest).rpartition('.')[0]

with TemporaryDirectory(prefix='razercommander-flatpak-repo') as temprepo:
    with TemporaryDirectory(prefix='razercommander-flatpak-build') as tempbuild:
        subprocess.call(['flatpak-builder', tempbuild, manifest, '--repo=' + temprepo])
    subprocess.call(['flatpak', 'build-bundle', temprepo, output, app_id])



#!/usr/bin/env python3

import os
import sys
import json

in_file = sys.argv[1]
git_repo = os.path.join(os.path.dirname(in_file), '.git')
output = sys.argv[2]

manifest = json.load(open(in_file, encoding='utf-8'))
manifest['modules'][0]['sources'][0]['url'] = git_repo
json.dump(manifest, open(output, 'w', encoding='utf-8'))



import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Gdk, GdkPixbuf

def make_image_row(text, img_path):
    box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
    label = Gtk.Label()
    label.set_text(text)
    icon = Gtk.Image()
    icon.set_from_resource(img_path)
    label.set_margin_left(12)
    label.set_margin_right(12)
    box.pack_start(icon, False, False, 0)
    box.pack_start(label, False, False, 0)
    row = Gtk.ListBoxRow()
    row.add(box)
    return row

def make_row(text):
    box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
    label = Gtk.Label()
    label.set_text(text)
    label.set_margin_left(12)
    label.set_margin_right(12)
    box.set_margin_top(12)
    box.set_margin_bottom(12)
    box.pack_start(label, False, False, 0)
    row = Gtk.ListBoxRow()
    row.add(box)
    return row

def make_2x_row(text1, text2):
    box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
    label1 = Gtk.Label()
    label1.set_text(text1)
    label1.set_margin_left(12)
    label1.set_margin_right(12)

    label2 = Gtk.Label()
    text2_for_label=text2
    if not text2_for_label:
        text2_for_label="Unassigned"
    label2.set_markup("<span color=\"#818181\">"+text2_for_label+"</span>")
    label2.set_margin_left(12)
    label2.set_margin_right(12)

    box.set_margin_top(12)
    box.set_margin_bottom(12)
    label1.set_size_request(200, 0)
    label2.set_size_request(200, 0)
    box.pack_start(label1, True, True, 0)
    box.pack_start(label2, True, False, 0)
    row = Gtk.ListBoxRow()
    row.add(box)
    row.value={'key': text1, 'val': text2}
    return row

def empty_listbox(listbox):
    while True:
        row = listbox.get_row_at_index(0)
        if row:
            listbox.remove(row)
        else:
            break



# probably not necessary vvvv
import os
import io
import logging
logging.basicConfig(level=logging.DEBUG)
from . import macro_logic

import openrazer.client as rclient
#import razer.client.constants as razer_constants

device_manager = rclient.DeviceManager()
devlist = []
for device in device_manager.devices:
    devlist.append(device)

def getSyncFX():
    return bool(device_manager.sync_effects)

def setSyncFX(value):
    if type(value) != bool:
        print('ERROR: device: setSyncFX: the value passed is not a boolean!')
        return False
    device_manager.sync_effects = value

# This class represents a single device (ie: a keyboard)
class Device:

    def __init__(self, device):
        self.device = device
        self.availableFX = []

        if self.device.type == 'mouse':
            for fx in self.mouse_scroll_uFXList:
                if self.device.fx.misc.has('scroll'):
                    if self.device.fx.misc.scroll_wheel.has(fx):
                        self.availableFX.append(fx)
            for fx in self.mouse_logo_uFXList:
                if self.device.fx.misc.has('logo'):
                    if self.device.fx.misc.logo.has(fx):
                        self.availableFX.append(fx)
        for fx in self.uFXList:
            if self.device.fx.has(fx):
                self.availableFX.append(fx)

        if self.device.has('lighting_led_matrix') and self.device.type == 'keyboard':
            self.availableFX.append('custom')
        if self.device.has('macro_logic'):  # unsupported dev failsafe in macro_logic.make_device()
            self.macro_device = macro_logic.make_device(
                str(self.device.serial),
                self.device)
        else:
            self.macro_device = None
        self.name = str(device.name)


    # unfriendly fx list
    uFXList = [
        'breath_single',
        'breath_dual',
        'breath_random',
        'reactive',
        'spectrum',
        'pulsate',
        'wave',
        'ripple',
        'static',
        'none',
    ]

    mouse_scroll_uFXList = [
        'scroll_blinking',
        'scroll_pulsate',
        'scroll_breath_single',
        'scroll_breath_dual',
        'scroll_breath_random',
        'scroll_spectrum',
        'scroll_reactive',
        'scroll_static',
        'scroll_none',
    ]

    mouse_logo_uFXList = [
        'logo_blinking',
        'logo_pulsate',
        'logo_breath_single',
        'logo_breath_dual',
        'logo_breath_random',
        'logo_spectrum',
        'logo_reactive',
        'logo_static',
        'logo_none',
    ]

    MSG_PROBLEM_ENABLING = 'There was an error enabling the FX '

    # Breathing effect mode
    def enableRandomBreath(self):
        # check if breathe is available for the current device
        if 'breath_random' in self.availableFX:
            if not self.device.fx.breath_random():
                logging.error(self.MSG_PROBLEM_ENABLING + 'Breath Random')
        else:
            logging.warning('The Breath Random FX is not available')

    def enableSingleBreath(self, R, G, B):
        # check if breathe is available for the current device
        if 'breath_single' in self.availableFX:
            if not self.device.fx.breath_single(R, G, B):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Breath Single')
        else:
            logging.warning('The Breath Single FX is not available')

    def enableDoubleBreath(self, R1, G1, B1, R2, G2, B2):
        # check if breathe is available for the current device
        if 'breath_dual' in self.availableFX:
            if not self.device.fx.breath_dual(R1, G1, B1, R2, G2, B2):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Breath Double')
        else:
            logging.warning('The Breath Double FX is not available')

    # Reactive effect mode
    # this method takes 3 arguments, the values for red, green and blue
    def enableReactive(self, time, R, G, B):  # time can be only 1, 2 or 3
        # check if reactive is available for the current device
        if 'reactive' in self.availableFX:
            if not self.device.fx.reactive(R, G, B, time):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Reactive')
        else:
            logging.warning('The Reactive FX is not available')

    # Static effect mode
    # this method takes 3 arguments, the values for red, green and blue
    def enableStatic(self, R, G, B):
        # check if static is available for the current device
        if 'static' in self.availableFX:
            if not self.device.fx.static(R, G, B):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Static')
        else:
            logging.warning('The Static FX is not available')

    # Wave effect mode
    # direction is an int: 1 is right, 2 is left
    def enableWave(self, direction):
        # check if wave is available for the current device
        if 'wave' in self.availableFX:
            if direction not in [1, 2]:
                logging.warning(
                    'Error when enabling Wave: value is not 1 or 2')
            elif not self.device.fx.wave(direction):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Wave')
        else:
            logging.warning('The Wave FX is not available')

    # NOTE: starlight mode FX temporarely disabled since it's not supported by the lib
    # Starlight mode effect
    # This is pretty easy but undocumented in the driver wiki, so
    # just be aware of that
    # def enableStarlight(self):
        # check if starlight is available for the current device
    #    if self.KNOWN_MODE_BUFFERS['STARLIGHT'] in self.mode_buffers:
    #        command='echo -n \'1\' > '+self.DRIVER_PATH+self.escaped_uid+'/'+self.KNOWN_MODE_BUFFERS['STARLIGHT']
    #        self.__gksu_run__(command)
    #        return 0
    #    else:
    #        self.__dbug_print__("Starlight not available on this device")
    #        return 1

    # None effect mode
    # Practically disables lighting effects
    def enableNone(self):
        # check if none is available for the current device
        if 'none' in self.availableFX:
            if not self.device.fx.none():
                logging.error(self.MSG_PROBLEM_ENABLING + 'None')
        else:
            logging.warning('The None FX is not available')

    # Turns on or off game mode (it disables the <Super> key)
    # Value has to be 0 to disable or 1 to enable game mode
    def toggleGameMode(self):
        # check if game is available for the current device
        if self.device.has('game_mode_led'):
            # toggles game mode
            self.device.game_mode_led = not self.device.game_mode_led
        else:
            logging.warning('The Game Mode LED is not available')

    # Spectrum mode effect
    # This should only be supported in RGB keyboards
    def enableSpectrum(self):
        # check if spectrum is available for the current device
        if 'spectrum' in self.availableFX:
            if not self.device.fx.spectrum():
                logging.error(self.MSG_PROBLEM_ENABLING + 'Spectrum')
        else:
            logging.warning('The Spectrum FX is not available')

    def enableRipple(self, R, G, B):
        if 'ripple' in self.availableFX:
            if not self.device.fx.ripple(
                R, G, B, rclient.constants.RIPPLE_REFRESH_RATE):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Ripple')
        else:
            logging.warning('The Ripple FX is not available')

    def enableRippleRandom(self):
        if 'ripple' in self.availableFX:
            if not self.device.fx.ripple_random(rclient.constants.RIPPLE_REFRESH_RATE):
                logging.error(self.MSG_PROBLEM_ENABLING + 'Ripple')
        else:
            logging.warning('The Ripple FX is not available')

    # Pulsate mode effect
    # This should only be supported in the Razer BlackWidow Ultimate 2013
    # and should be similar if not the same as breath effect
    def enablePulsate(self):
        # check if pulsate is available for the current device
        if 'pulsate' in self.availableFX:
            if not self.device.fx.pulsate():
                logging.error(self.MSG_PROBLEM_ENABLING + 'Pulsate')
        else:
            logging.warning('The Pulsate FX is not available')


    def getBrightness(self):
        # check if brightness is available for the current device
        if self.device.has('brightness'):
            return self.device.brightness
        else:
            return -1
            logging.warning('Brightness is not available')

    # Set birghtness for backlight
    # gets values between 0 and 255
    def setBrightness(self, value):
        # check if brightness is available for the current device
        if self.device.has('brightness'):
            self.device.brightness = value
        else:
            logging.warning('Brightness is not available')

    def getKbBrightness(self):
        if self.device.has('brightness'):
            return int(self.device.brightness)
        else:
            logging.warning('Brightness is not available')

    def getScrollBrightness(self):
        # check if brightness is available for the current device
        if self.device.fx.has('scroll_brightness'):
            return self.device.fx.misc.scroll_wheel.brightness
        else:
            logging.warning('Scroll Brightness is not available')
            return -1

    def getLogoBrightness(self):
        # check if brightness is available for the current device
        if self.device.fx.has('logo_brightness'):
            return self.device.fx.misc.logo.brightness
        else:
            logging.warning('Logo Brightness is not available')
            return -1

    def setScrollBrightness(self, value):
        # check if brightness is available for the current device
        if self.device.fx.has('scroll_brightness'):
            self.device.fx.misc.scroll_wheel.brightness = value
        else:
            logging.warning('Scroll Brightness is not available')

    def setLogoBrightness(self, value):
        # check if brightness is available for the current device
        if self.device.fx.has('logo_brightness'):
            self.device.fx.misc.logo.brightness = value
        else:
            logging.warning('Logo Brightness is not available')

    def enableFX(self, fx):
        if fx == "None":
            self.enableNone()
            return 0
        elif fx == "Spectrum":
            self.enableSpectrum()
            return 0
        elif fx == "Pulsate":
            self.enablePulsate()
            return 0
        elif fx == 'Scroll spectrum':
            self.enableScrollSpectrum()
        elif fx == 'Logo spectrum':
            self.enableLogoSpectrum()
        elif fx == 'Scroll none':
            self.enableScrollNone()
        elif fx == 'Logo none':
            self.enableLogoNone()
        else:
            logging.error("FX not listed")
            return 1

    def enableScrollBlinking(self, R, G, B):
        if self.device.fx.misc.scroll_wheel.has('scroll_blinking'):
            if not self.device.fx.misc.scroll_wheel.blinking(R, G, B):
                logging.error('{}Blinking'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Scroll Blinking FX is not available')

    def enableScrollPulsate(self, R, G, B):
        if self.device.fx.misc.scroll_wheel.has('scroll_pulsate'):
            if not self.device.fx.misc.scroll_wheel.pulsate(R, G, B):
                logging.error('{}Pulsate'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Scroll Pulsate FX is not available')

    def enableScrollBreathSingle(self, R, G, B):
        if self.device.fx.misc.scroll_wheel.has('scroll_breath_single'):
            if not self.device.fx.misc.scroll_wheel.breath_single(R, G, B):
                logging.error('{}Breath Single'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Breath Single FX is not available')

    def enableScrollBreathDual(self, R, G, B, R1, G1, B1):
        if self.device.fx.misc.scroll_wheel.has('scroll_breath_dual'):
            if not self.device.fx.misc.scroll_wheel.breath_dual(R, G, B, R1, G1, B1):
                logging.error('{}Breath Dual'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Breath Dual FX is not available')

    def enableScrollBreathRandom(self):
        if self.device.fx.misc.scroll_wheel.has('scroll_breath_random'):
            if not self.device.fx.misc.scroll_wheel.breath_random():
                logging.error('{}Breath Random'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Breath Random FX is not available')

    def enableScrollSpectrum(self):
        if self.device.fx.misc.scroll_wheel.has('scroll_spectrum'):
            if not self.device.fx.misc.scroll_wheel.spectrum():
                logging.error('{}Spectrum'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Spectrum FX is not available')

    def enableScrollReactive(self, R, G, B, time):
        if self.device.fx.misc.scroll_wheel.has('scroll_reactive'):
            if not self.device.fx.misc.scroll_wheel.reactive(R, G, B, time):
                logging.error('{}Reactive'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Reactive FX is not available')

    def enableScrollStatic(self, R, G, B):
        if self.device.fx.misc.scroll_wheel.has('scroll_static'):
            if not self.device.fx.misc.scroll_wheel.static(R, G, B):
                logging.error('{}Static'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Static FX is not available')

    def enableScrollNone(self):
        if self.device.fx.misc.scroll_wheel.has('scroll_none'):
            if not self.device.fx.misc.scroll_wheel.none():
                logging.error('{}None'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The None FX is not available')

    def enableLogoBlinking(self, R, G, B):
        if self.device.fx.misc.logo.has('logo_blinking'):
            if not self.device.fx.misc.logo.blinking(R, G, B):
                logging.error('{}Blinking'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Logo Blinking FX is not available')

    def enableLogoPulsate(self, R, G, B):
        if self.device.fx.misc.logo.has('logo_pulsate'):
            if not self.device.fx.misc.logo.pulsate(R, G, B):
                logging.error('{}Pulsate'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Logo Pulsate FX is not available')

    def enableLogoBreathSingle(self, R, G, B):
        if self.device.fx.misc.logo.has('logo_breath_single'):
            if not self.device.fx.misc.logo.breath_single(R, G, B):
                logging.error('{}Breath Single'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Breath Single FX is not available')

    def enableLogoBreathDual(self, R, G, B, R1, G1, B1):
        if self.device.fx.misc.logo.has('logo_breath_dual'):
            if not self.device.fx.misc.logo.breath_dual(R, G, B, R1, G1, B1):
                logging.error('{}Breath Dual'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Breath Dual FX is not available')

    def enableLogoBreathRandom(self):
        if self.device.fx.misc.logo.has('logo_breath_random'):
            if not self.device.fx.misc.logo.breath_random():
                logging.error('{}Breath Random'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Breath Random FX is not available')

    def enableLogoSpectrum(self):
        if self.device.fx.misc.logo.has('logo_spectrum'):
            if not self.device.fx.misc.logo.spectrum():
                logging.error('{}Spectrum'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Spectrum FX is not available')

    def enableLogoReactive(self, R, G, B, time):
        if self.device.fx.misc.logo.has('logo_reactive'):
            if not self.device.fx.misc.logo.reactive(R, G, B, time):
                logging.error('{}Reactive'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Reactive FX is not available')

    def enableLogoStatic(self, R, G, B):
        if self.device.fx.misc.logo.has('logo_static'):
            if not self.device.fx.misc.logo.static(R, G, B):
                logging.error('{}Static'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The Static FX is not available')

    def enableLogoNone(self):
        if self.device.fx.misc.logo.has('logo_none'):
            if not self.device.fx.misc.logo.none():
                logging.error('{}None'.format(self.MSG_PROBLEM_ENABLING))
        else:
            logging.warning('The None FX is not available')

    def _make_color_tuple(self, mcol):
    	return (int(mcol.red*255), int(mcol.green*255), int(mcol.blue*255))

    def set_macro(self, key, command):
        if not self.macro_device:
            print("ERROR: device "+self.name+" doesn't implement macro logic")
            return
        self.macro_device.set_macro(key, command)

    #legacy
    def assignMacro(self, key, command):
        script_macro=self.device.macro.create_script_macro_item(command)
        self.device.macro.add_macro(key, script_macro)

    def applyCustom(self, customKb):
        rindex = 0
        for row in customKb.rows:
            cindex=0
            for key in row.keylist:
                # print(self._make_color_tuple(key.color))
                self.device.fx.advanced.matrix.set(
                    rindex, cindex, self._make_color_tuple(key.color)
                )
                cindex+=1
            rindex += 1
        self.device.fx.advanced.draw()

    def get_poll_rate(self):
        if self.device.has('poll_rate'):
            return self.device.poll_rate
        else:
            return -1

    def set_poll_rate(self, value):
        # the poll rate value can only be 125, 500 or 1000
        if not value in [125,500,1000]:
            print('ERROR: Poll rate can only be one of 125, 500 or 1000')
            return
        if not self.device.has('poll_rate'):
            print('ERROR: device {} doesn\'t support poll_rate'.format(self.device.name))
            return
        self.device.poll_rate = value

    def get_dpi(self):
        if self.device.has('dpi'):
            return self.device.dpi
        else:
            return -1

    def get_max_dpi(self):
        if self.device.has('dpi'):
            return self.device.max_dpi
        else:
            return -1

    def set_dpi(self, dpi1, dpi2):
        if not self.device.has('dpi'):
            print('ERROR: device {} doesn\'t support dpi'.format(self.device.name))
            return
        if dpi1 > self.device.max_dpi or dpi2 > self.device.max_dpi:
            print('ERROR: the dpi value(s) provided are over the maximum dpi supported by the device')
            return
        self.device.dpi = (dpi1, dpi2)



CONF_FILE_PATH='macro_logic.json'
from .devicesMacroKeys import *
from . import listboxHelper
from . import confFileManager
macros_conf_loaded={}
confFileManager.check_path(CONF_FILE_PATH)
macros_conf_loaded=confFileManager.load_file(CONF_FILE_PATH, macros_conf_loaded)

class MacroDevice:
    def __init__(self, macrokey_arr, device_uid, device):
        self.macrokey_arr = macrokey_arr
        self.device_uid = device_uid
        self.device = device
        self.set_all_current_macros()

    def get_key(self, key_n):
        for k in self.macrokey_arr:
            if key_n == k.key:
                return k
        return None

    def set_macro(self, key, n_macro):
        key_obj=self.get_key(key)
        if not key_obj:
            raise ValueError("get_key() for key "+key+" returned None")
        macros_conf_loaded[self.device_uid][key]=n_macro
        key_obj._set_macro(self.device, n_macro)
        confFileManager.save_file(CONF_FILE_PATH, macros_conf_loaded)

    def set_all_current_macros(self):
        for key in self.macrokey_arr:
            key.set_current_macro(self.device)


class MacroKey:
    def __init__(self, key, macro=None):
        self.key = key
        self.macro = macro

    def _set_macro(self, device, n_macro):
        self.macro=n_macro
        if not n_macro:
            device.macro.del_macro(self.key)
            return
        script_macro=device.macro.create_script_macro_item(n_macro)
        # add macro doesn't append macros to the same key, it overwrites
        device.macro.add_macro(self.key, script_macro)

    def set_current_macro(self, device):
        self._set_macro(device, self.macro)

    def __repr__(self):
        return self.key


def make_shortcutslistbox_rows(m_dev):
    rowlist=[]
    for key in m_dev.macrokey_arr:
        rowlist.append(listboxHelper.make_2x_row(key.key, key.macro))
    return rowlist

def make_device(device_uid, device):
    try:
        if not device.name in macro_kls.keys(): # failsafe for unsupported devices
            return None
        mk_arr=[]
        if not device_uid in macros_conf_loaded.keys():
            macros_conf_loaded[device_uid] = dict()
            for key_n in macro_kls[device.name]:
                macros_conf_loaded[device_uid][key_n] = ''
            confFileManager.save_file(CONF_FILE_PATH, macros_conf_loaded)
        for key_n in macro_kls[device.name]:
            mk_arr.append(MacroKey(key_n, macros_conf_loaded[device_uid][key_n]))
        n_device = MacroDevice(mk_arr, device_uid, device)
        n_device.set_all_current_macros()
        return n_device
    except Exception as ex:
        print('Skipping macro logic for device {}\n  Exception: {}'.format(device.name, ex))
        return None
